@@ -8,23 +8,7 @@
  * Copyright (c) 2003, Frank Warmerdam
  * Copyright (c) 2008, Even Rouault <even dot rouault at spatialys.com>
  *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be included
- * in all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
- * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
- * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS IN THE SOFTWARE.
+ * SPDX-License-Identifier: MIT
  ****************************************************************************/
 
 #include <wchar.h>
@@ -193,7 +177,7 @@ bool CPLODBCDriverInstaller::FindMdbToolsDriverLib(CPLString &osDriverFile)
         if (LibraryExists(strLibPath.c_str()))
         {
             // Save custom driver path
-            osDriverFile = strLibPath;
+            osDriverFile = std::move(strLibPath);
             return true;
         }
     }
@@ -278,7 +262,7 @@ bool CPLODBCDriverInstaller::LibraryExists(const char *pszLibPath)
 
 void CPLODBCDriverInstaller::InstallMdbToolsDriver()
 {
-#ifdef WIN32
+#ifdef _WIN32
     return;
 #else
     static std::once_flag oofDriverInstallAttempted;
@@ -613,13 +597,22 @@ bool CPLODBCSession::ConnectToMsAccess(const char *pszName,
     const auto Connect =
         [this, &pszName](const char *l_pszDSNStringTemplate, bool bVerboseError)
     {
-        char *pszDSN = static_cast<char *>(
-            CPLMalloc(strlen(pszName) + strlen(l_pszDSNStringTemplate) + 100));
-        /* coverity[tainted_string] */
-        snprintf(pszDSN, strlen(pszName) + strlen(l_pszDSNStringTemplate) + 100,
-                 l_pszDSNStringTemplate, pszName);
-        CPLDebug("ODBC", "EstablishSession(%s)", pszDSN);
-        int bError = !EstablishSession(pszDSN, nullptr, nullptr);
+        std::string osDSN;
+        constexpr const char *PCT_S = "%s";
+        const char *pszPctS = strstr(l_pszDSNStringTemplate, PCT_S);
+        if (!pszPctS)
+        {
+            osDSN = l_pszDSNStringTemplate;
+        }
+        else
+        {
+            osDSN.assign(l_pszDSNStringTemplate,
+                         pszPctS - l_pszDSNStringTemplate);
+            osDSN += pszName;
+            osDSN += (pszPctS + strlen(PCT_S));
+        }
+        CPLDebug("ODBC", "EstablishSession(%s)", osDSN.c_str());
+        int bError = !EstablishSession(osDSN.c_str(), nullptr, nullptr);
         if (bError)
         {
             if (bVerboseError)
@@ -627,13 +620,11 @@ bool CPLODBCSession::ConnectToMsAccess(const char *pszName,
                 CPLError(CE_Failure, CPLE_AppDefined,
                          "Unable to initialize ODBC connection to DSN for %s,\n"
                          "%s",
-                         pszDSN, GetLastError());
+                         osDSN.c_str(), GetLastError());
             }
-            CPLFree(pszDSN);
             return false;
         }
 
-        CPLFree(pszDSN);
         return true;
     };
 
@@ -710,6 +701,16 @@ int CPLODBCSession::EstablishSession(const char *pszDSN, const char *pszUserid,
     if (pszPassword == nullptr)
         pszPassword = "";
 
+    std::string osDSN(pszDSN);
+#if defined(_WIN32)
+    if (CPLTestBool(CPLGetConfigOption("GDAL_FILENAME_IS_UTF8", "YES")))
+    {
+        char *pszTemp = CPLRecode(pszDSN, CPL_ENC_UTF8, "CP_ACP");
+        osDSN = pszTemp;
+        CPLFree(pszTemp);
+    }
+#endif
+
     bool bFailed = false;
     if (strstr(pszDSN, "=") != nullptr)
     {
@@ -719,7 +720,7 @@ int CPLODBCSession::EstablishSession(const char *pszDSN, const char *pszUserid,
 
         bFailed = CPL_TO_BOOL(Failed(SQLDriverConnect(
             m_hDBC, nullptr,
-            reinterpret_cast<SQLCHAR *>(const_cast<char *>(pszDSN)),
+            reinterpret_cast<SQLCHAR *>(const_cast<char *>(osDSN.c_str())),
             static_cast<SQLSMALLINT>(strlen(pszDSN)), szOutConnString,
             sizeof(szOutConnString), &nOutConnStringLen, SQL_DRIVER_NOPROMPT)));
     }
@@ -727,7 +728,8 @@ int CPLODBCSession::EstablishSession(const char *pszDSN, const char *pszUserid,
     {
         CPLDebug("ODBC", "SQLConnect(%s)", pszDSN);
         bFailed = CPL_TO_BOOL(Failed(SQLConnect(
-            m_hDBC, reinterpret_cast<SQLCHAR *>(const_cast<char *>(pszDSN)),
+            m_hDBC,
+            reinterpret_cast<SQLCHAR *>(const_cast<char *>(osDSN.c_str())),
             SQL_NTS, reinterpret_cast<SQLCHAR *>(const_cast<char *>(pszUserid)),
             SQL_NTS,
             reinterpret_cast<SQLCHAR *>(const_cast<char *>(pszPassword)),
@@ -1599,6 +1601,7 @@ int CPLODBCStatement::Failed(int nResultCode)
 
     return TRUE;
 }
+
 //! @endcond
 
 /************************************************************************/
@@ -1637,6 +1640,24 @@ void CPLODBCStatement::Append(const char *pszText)
     m_nStatementLen += nTextLen;
 }
 
+/************************************************************************/
+/*                      Append(const std::string &)                     */
+/************************************************************************/
+
+/**
+ * Append text to internal command.
+ *
+ * The passed text is appended to the internal SQL command text.
+ *
+ * @param s text to append.
+ */
+
+void CPLODBCStatement::Append(const std::string &s)
+
+{
+    Append(s.c_str());
+}
+
 /************************************************************************/
 /*                     AppendEscaped(const char *)                      */
 /************************************************************************/
